/* (C) 2021 YiRing, Inc. */
package com.yiring.app.web;

import cn.hutool.extra.spring.SpringUtil;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import com.yiring.app.constant.Code;
import com.yiring.app.domain.TestTable;
import com.yiring.app.exception.CodeException;
import com.yiring.app.mapper.TestTableMapper;
import com.yiring.common.core.Result;
import com.yiring.common.param.PageParam;
import com.yiring.common.util.FileUtils;
import com.yiring.common.vo.PageVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import java.io.IOException;
import java.util.Arrays;
import java.util.List;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import lombok.extern.slf4j.Slf4j;
import org.springframework.core.io.ClassPathResource;
import org.springframework.http.MediaType;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

@Slf4j
@Validated
@SuppressWarnings({ "deprecation" })
@ApiSupport(order = 0)
@Api(tags = "示例", description = "Example")
@RequestMapping("/example/")
@RestController
public class ExampleController {

    String text = "😎 Hello World";

    @GetMapping
    public Result<String> hello() {
        return Result.ok(text);
    }

    /**
     * 测试失败自定义状态信息输出
     */
    @GetMapping("fail")
    public Result<String> fail() {
        throw new CodeException(Code.FAIL);
    }

    @GetMapping("page")
    public Result<PageVo<String>> page(@Valid PageParam pageParam) {
        log.info("PageParam: {}", pageParam);

        List<String> data = Arrays.asList(text.split(" "));
        PageVo<String> vo = PageVo.build(data, data.size());
        return Result.ok(vo);
    }

    @ApiOperation(value = "download", produces = MediaType.APPLICATION_OCTET_STREAM_VALUE)
    @GetMapping("download")
    public void download(HttpServletResponse response) throws IOException {
        ClassPathResource resource = new ClassPathResource("static/cat.jpg");
        FileUtils.download(response, resource.getFile());
    }

    @ApiOperation("测试 MyBatis Plus 查询")
    @GetMapping("test")
    public Result<String> test() {
        List<TestTable> tests = SpringUtil.getBean(TestTableMapper.class).selectList(null);
        return Result.ok(tests.stream().map(TestTable::getName).reduce((a, b) -> a + "," + b).orElse(""));
    }
}
